# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

import os
import sys
from unittest.mock import Mock

sys.path.insert(0, os.path.abspath("../../"))
sys.path.insert(0, os.path.abspath("../../game_templates"))


# -- Project information -----------------------------------------------------

project = "SurroRTG SDK"
copyright = "2020, Surrogate.tv"
author = "Surrogate.tv"

# The full version, including alpha/beta/rc tags
release = "0.1"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    # "sphinx.ext.viewcode",
    "myst_parser",
]

source_suffix = {
    ".rst": "restructuredtext",
    ".txt": "markdown",
    ".md": "markdown",
}

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# remove full module paths
add_module_names = False

# disable default alphabetical method order
autodoc_member_order = "bysource"

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"
html_show_sourcelink = False

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]
html_css_files = [
    "css/custom.css",
]
html_theme_options = {
    "logo_only": True,
    "collapse_navigation": False,
}
html_logo = "_static/images/logo.png"

# this fixes logo flashing, remove when fix on master:
# https://github.com/readthedocs/sphinx_rtd_theme/issues/724
html_js_files = [
    "js/modernizr.min.js",
]

# -- Custom -----------------------------------------------------------------


def import_mock_modules(names):
    for name in names:
        sys.modules[name] = Mock()


# fix import warnings
import_mock_modules(
    [
        "adafruit_tcs34725",
        "cv2",
        "board",
        "busio",
        "pigpio",
        "socketio",
        "toml",
    ]
)
